@@ -31,12 +31,7 @@ impl<T: Transport> Accounts<T> {
     where
         S: AsRef<[u8]>,
     {
-        let message = message.as_ref();
-
-        let mut eth_message = format!("\x19Ethereum Signed Message:\n{}", message.len()).into_bytes();
-        eth_message.extend_from_slice(message);
-
-        signing::keccak256(&eth_message).into()
+        signing::hash_message(message)
     }
 }
 
@@ -48,12 +43,17 @@ mod accounts_signing {
         error,
         signing::Signature,
         types::{
-            Address, Bytes, Recovery, RecoveryMessage, SignedData, SignedTransaction, TransactionParameters, U256,
+            AccessList, Address, Bytes, Recovery, RecoveryMessage, SignedData, SignedTransaction,
+            TransactionParameters, U256, U64,
         },
     };
     use rlp::RlpStream;
     use std::convert::TryInto;
 
+    const LEGACY_TX_ID: u64 = 0;
+    const ACCESSLISTS_TX_ID: u64 = 1;
+    const EIP1559_TX_ID: u64 = 2;
+
     impl<T: Transport> Accounts<T> {
         /// Gets the parent `web3` namespace
         fn web3(&self) -> Web3<T> {
@@ -82,13 +82,29 @@ mod accounts_signing {
                 };
             }
             let from = key.address();
+
+            let gas_price = match tx.transaction_type {
+                Some(tx_type) if tx_type == U64::from(EIP1559_TX_ID) && tx.max_fee_per_gas.is_some() => {
+                    tx.max_fee_per_gas
+                }
+                _ => tx.gas_price,
+            };
+
             let (nonce, gas_price, chain_id) = futures::future::try_join3(
                 maybe!(tx.nonce, self.web3().eth().transaction_count(from, None)),
-                maybe!(tx.gas_price, self.web3().eth().gas_price()),
+                maybe!(gas_price, self.web3().eth().gas_price()),
                 maybe!(tx.chain_id.map(U256::from), self.web3().eth().chain_id()),
             )
             .await?;
             let chain_id = chain_id.as_u64();
+
+            let max_priority_fee_per_gas = match tx.transaction_type {
+                Some(tx_type) if tx_type == U64::from(EIP1559_TX_ID) => {
+                    tx.max_priority_fee_per_gas.unwrap_or(gas_price)
+                }
+                _ => gas_price,
+            };
+
             let tx = Transaction {
                 to: tx.to,
                 nonce,
@@ -96,7 +112,11 @@ mod accounts_signing {
                 gas_price,
                 value: tx.value,
                 data: tx.data.0,
+                transaction_type: tx.transaction_type,
+                access_list: tx.access_list.unwrap_or_default(),
+                max_priority_fee_per_gas,
             };
+
             let signed = tx.sign(key, chain_id);
             Ok(signed)
         }
@@ -116,7 +136,7 @@ mod accounts_signing {
             let message_hash = self.hash_message(message);
 
             let signature = key
-                .sign(&message_hash.as_bytes(), None)
+                .sign(message_hash.as_bytes(), None)
                 .expect("hash is non-zero 32-bytes; qed");
             let v = signature
                 .v
@@ -159,12 +179,13 @@ mod accounts_signing {
             };
             let (signature, recovery_id) = recovery
                 .as_signature()
-                .ok_or_else(|| error::Error::Recovery(signing::RecoveryError::InvalidSignature))?;
+                .ok_or(error::Error::Recovery(signing::RecoveryError::InvalidSignature))?;
             let address = signing::recover(message_hash.as_bytes(), &signature, recovery_id)?;
             Ok(address)
         }
     }
     /// A transaction used for RLP encoding, hashing and signing.
+    #[derive(Debug)]
     pub struct Transaction {
         pub to: Option<Address>,
         pub nonce: U256,
@@ -172,67 +193,158 @@ mod accounts_signing {
         pub gas_price: U256,
         pub value: U256,
         pub data: Vec<u8>,
+        pub transaction_type: Option<U64>,
+        pub access_list: AccessList,
+        pub max_priority_fee_per_gas: U256,
     }
 
     impl Transaction {
-        /// RLP encode an unsigned transaction for the specified chain ID.
-        fn rlp_append_unsigned(&self, rlp: &mut RlpStream, chain_id: u64) {
-            rlp.begin_list(9);
-            rlp.append(&self.nonce);
-            rlp.append(&self.gas_price);
-            rlp.append(&self.gas);
+        fn rlp_append_legacy(&self, stream: &mut RlpStream) {
+            stream.append(&self.nonce);
+            stream.append(&self.gas_price);
+            stream.append(&self.gas);
             if let Some(to) = self.to {
-                rlp.append(&to);
+                stream.append(&to);
             } else {
-                rlp.append(&"");
+                stream.append(&"");
             }
-            rlp.append(&self.value);
-            rlp.append(&self.data);
-            rlp.append(&chain_id);
-            rlp.append(&0u8);
-            rlp.append(&0u8);
+            stream.append(&self.value);
+            stream.append(&self.data);
         }
 
-        /// RLP encode a signed transaction with the specified signature.
-        fn rlp_append_signed(&self, rlp: &mut RlpStream, signature: &Signature) {
-            rlp.begin_list(9);
-            rlp.append(&self.nonce);
-            rlp.append(&self.gas_price);
-            rlp.append(&self.gas);
-            if let Some(to) = self.to {
-                rlp.append(&to);
+        fn encode_legacy(&self, chain_id: u64, signature: Option<&Signature>) -> RlpStream {
+            let mut stream = RlpStream::new();
+            stream.begin_list(9);
+
+            self.rlp_append_legacy(&mut stream);
+
+            if let Some(signature) = signature {
+                self.rlp_append_signature(&mut stream, signature);
             } else {
-                rlp.append(&"");
+                stream.append(&chain_id);
+                stream.append(&0u8);
+                stream.append(&0u8);
+            }
+
+            stream
+        }
+
+        fn encode_access_list_payload(&self, chain_id: u64, signature: Option<&Signature>) -> RlpStream {
+            let mut stream = RlpStream::new();
+
+            let list_size = if signature.is_some() { 11 } else { 8 };
+            stream.begin_list(list_size);
+
+            // append chain_id. from EIP-2930: chainId is defined to be an integer of arbitrary size.
+            stream.append(&chain_id);
+
+            self.rlp_append_legacy(&mut stream);
+            self.rlp_append_access_list(&mut stream);
+
+            if let Some(signature) = signature {
+                self.rlp_append_signature(&mut stream, signature);
+            }
+
+            stream
+        }
+
+        fn encode_eip1559_payload(&self, chain_id: u64, signature: Option<&Signature>) -> RlpStream {
+            let mut stream = RlpStream::new();
+
+            let list_size = if signature.is_some() { 12 } else { 9 };
+            stream.begin_list(list_size);
+
+            // append chain_id. from EIP-2930: chainId is defined to be an integer of arbitrary size.
+            stream.append(&chain_id);
+
+            stream.append(&self.nonce);
+            stream.append(&self.max_priority_fee_per_gas);
+            stream.append(&self.gas_price);
+            stream.append(&self.gas);
+            if let Some(to) = self.to {
+                stream.append(&to);
+            } else {
+                stream.append(&"");
+            }
+            stream.append(&self.value);
+            stream.append(&self.data);
+
+            self.rlp_append_access_list(&mut stream);
+
+            if let Some(signature) = signature {
+                self.rlp_append_signature(&mut stream, signature);
+            }
+
+            stream
+        }
+
+        fn rlp_append_signature(&self, stream: &mut RlpStream, signature: &Signature) {
+            stream.append(&signature.v);
+            stream.append(&U256::from_big_endian(signature.r.as_bytes()));
+            stream.append(&U256::from_big_endian(signature.s.as_bytes()));
+        }
+
+        fn rlp_append_access_list(&self, stream: &mut RlpStream) {
+            stream.begin_list(self.access_list.len());
+            for access in self.access_list.iter() {
+                stream.begin_list(2);
+                stream.append(&access.address);
+                stream.begin_list(access.storage_keys.len());
+                for storage_key in access.storage_keys.iter() {
+                    stream.append(storage_key);
+                }
+            }
+        }
+
+        fn encode(&self, chain_id: u64, signature: Option<&Signature>) -> Vec<u8> {
+            match self.transaction_type.map(|t| t.as_u64()) {
+                Some(LEGACY_TX_ID) | None => {
+                    let stream = self.encode_legacy(chain_id, signature);
+                    stream.out().to_vec()
+                }
+
+                Some(ACCESSLISTS_TX_ID) => {
+                    let tx_id: u8 = ACCESSLISTS_TX_ID as u8;
+                    let stream = self.encode_access_list_payload(chain_id, signature);
+                    [&[tx_id], stream.as_raw()].concat()
+                }
+
+                Some(EIP1559_TX_ID) => {
+                    let tx_id: u8 = EIP1559_TX_ID as u8;
+                    let stream = self.encode_eip1559_payload(chain_id, signature);
+                    [&[tx_id], stream.as_raw()].concat()
+                }
+
+                _ => {
+                    panic!("Unsupported transaction type");
+                }
             }
-            rlp.append(&self.value);
-            rlp.append(&self.data);
-            rlp.append(&signature.v);
-            rlp.append(&U256::from_big_endian(signature.r.as_bytes()));
-            rlp.append(&U256::from_big_endian(signature.s.as_bytes()));
         }
 
         /// Sign and return a raw signed transaction.
         pub fn sign(self, sign: impl signing::Key, chain_id: u64) -> SignedTransaction {
-            let mut rlp = RlpStream::new();
-            self.rlp_append_unsigned(&mut rlp, chain_id);
+            let adjust_v_value = matches!(self.transaction_type.map(|t| t.as_u64()), Some(LEGACY_TX_ID) | None);
 
-            let hash = signing::keccak256(rlp.as_raw());
-            let signature = sign
-                .sign(&hash, Some(chain_id))
-                .expect("hash is non-zero 32-bytes; qed");
+            let encoded = self.encode(chain_id, None);
 
-            rlp.clear();
-            self.rlp_append_signed(&mut rlp, &signature);
+            let hash = signing::keccak256(encoded.as_ref());
 
-            let transaction_hash = signing::keccak256(rlp.as_raw()).into();
-            let raw_transaction = rlp.out().to_vec().into();
+            let signature = if adjust_v_value {
+                sign.sign(&hash, Some(chain_id))
+                    .expect("hash is non-zero 32-bytes; qed")
+            } else {
+                sign.sign_message(&hash).expect("hash is non-zero 32-bytes; qed")
+            };
+
+            let signed = self.encode(chain_id, Some(&signature));
+            let transaction_hash = signing::keccak256(signed.as_ref()).into();
 
             SignedTransaction {
                 message_hash: hash.into(),
                 v: signature.v,
                 r: signature.r,
                 s: signature.s,
-                raw_transaction,
+                raw_transaction: signed.into(),
                 transaction_hash,
             }
         }
@@ -431,6 +543,9 @@ mod tests {
             to: Some(hex!("F0109fC8DF283027b6285cc889F5aA624EaC1F55").into()),
             value: 1_000_000_000.into(),
             data: Vec::new(),
+            transaction_type: None,
+            access_list: vec![],
+            max_priority_fee_per_gas: 0.into(),
         };
         let skey = SecretKey::from_slice(&hex!(
             "4c0883a69102937d6231471b5dbb6204fe5129617082792ae468d01a3f362318"
