@@ -33,6 +33,12 @@ pub struct CallRequest {
     /// Access list
     #[serde(rename = "accessList", default, skip_serializing_if = "Option::is_none")]
     pub access_list: Option<AccessList>,
+    /// Max fee per gas
+    #[serde(rename = "maxFeePerGas", skip_serializing_if = "Option::is_none")]
+    pub max_fee_per_gas: Option<U256>,
+    /// miner bribe
+    #[serde(rename = "maxPriorityFeePerGas", skip_serializing_if = "Option::is_none")]
+    pub max_priority_fee_per_gas: Option<U256>,
 }
 
 impl CallRequest {
@@ -43,7 +49,7 @@ impl CallRequest {
 }
 
 /// Call Request Builder
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, Default)]
 pub struct CallRequestBuilder {
     call_request: CallRequest,
 }
@@ -143,6 +149,12 @@ pub struct TransactionRequest {
     /// Access list
     #[serde(rename = "accessList", default, skip_serializing_if = "Option::is_none")]
     pub access_list: Option<AccessList>,
+    /// Max fee per gas
+    #[serde(rename = "maxFeePerGas", skip_serializing_if = "Option::is_none")]
+    pub max_fee_per_gas: Option<U256>,
+    /// miner bribe
+    #[serde(rename = "maxPriorityFeePerGas", skip_serializing_if = "Option::is_none")]
+    pub max_priority_fee_per_gas: Option<U256>,
 }
 
 impl TransactionRequest {
@@ -153,7 +165,7 @@ impl TransactionRequest {
 }
 
 /// Transaction Request Builder
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, Default)]
 pub struct TransactionRequestBuilder {
     transaction_request: TransactionRequest,
 }
@@ -257,6 +269,8 @@ mod tests {
             data: Some(hex!("010203").into()),
             transaction_type: None,
             access_list: None,
+            max_fee_per_gas: None,
+            max_priority_fee_per_gas: None,
         };
 
         // when
@@ -282,7 +296,7 @@ mod tests {
   "value": "0x4c4b40",
   "data": "0x010203"
 }"#;
-        let deserialized: CallRequest = serde_json::from_str(&serialized).unwrap();
+        let deserialized: CallRequest = serde_json::from_str(serialized).unwrap();
 
         assert_eq!(deserialized.from, None);
         assert_eq!(deserialized.to, Some(Address::from_low_u64_be(5)));
@@ -306,6 +320,8 @@ mod tests {
             condition: Some(TransactionCondition::Block(5)),
             transaction_type: None,
             access_list: None,
+            max_fee_per_gas: None,
+            max_priority_fee_per_gas: None,
         };
 
         // when
@@ -337,7 +353,7 @@ mod tests {
     "block": 5
   }
 }"#;
-        let deserialized: TransactionRequest = serde_json::from_str(&serialized).unwrap();
+        let deserialized: TransactionRequest = serde_json::from_str(serialized).unwrap();
 
         assert_eq!(deserialized.from, Address::from_low_u64_be(5));
         assert_eq!(deserialized.to, None);
@@ -371,6 +387,8 @@ mod tests {
             data: Some(hex!("010203").into()),
             transaction_type: None,
             access_list: None,
+            max_fee_per_gas: None,
+            max_priority_fee_per_gas: None,
         };
         //when
         let call_request_builder = CallRequestBuilder::new()
@@ -407,6 +425,8 @@ mod tests {
             condition: Some(TransactionCondition::Block(5)),
             transaction_type: None,
             access_list: None,
+            max_fee_per_gas: None,
+            max_priority_fee_per_gas: None,
         };
         //when
         let tx_request_builder = TransactionRequestBuilder::new()
